livejs = '''
 /*
  Live.js - One script closer to Designing in the Browser
  Written for Handcraft.com by Martin Kool (@mrtnkl).

  Version 4.
  Recent change: Made stylesheet and mimetype checks case insensitive.

  http://livejs.com
  http://livejs.com/license (MIT)
  @livejs

  Include live.js#css to monitor css changes only.
  Include live.js#js to monitor js changes only.
  Include live.js#html to monitor html changes only.
  Mix and match to monitor a preferred combination such as live.js#html,css

  By default, just include live.js to monitor all css, js and html changes.

  Live.js can also be loaded as a bookmarklet. It is best to only use it for CSS then,
  as a page reload due to a change in html or css would not re-include the bookmarklet.
  To monitor CSS and be notified that it has loaded, include it as: live.js#css,notify
*/
(function () {

  var headers = { "Etag": 1, "Last-Modified": 1, "Content-Length": 1, "Content-Type": 1 },
      resources = {},
      pendingRequests = {},
      currentLinkElements = {},
      oldLinkElements = {},
      interval = 1000,
      loaded = false,
      active = { "html": 1, "css": 1, "js": 1 };

  var Live = {

    // performs a cycle per interval
    heartbeat: function () {
      if (document.body) {
        // make sure all resources are loaded on first activation
        if (!loaded) Live.loadresources();
        Live.checkForChanges();
      }
      setTimeout(Live.heartbeat, interval);
    },

    // loads all local css and js resources upon first activation
    loadresources: function () {

      // helper method to assert if a given url is local
      function isLocal(url) {
        var loc = document.location,
            reg = new RegExp("^\\.|^\/(?!\/)|^[\\w]((?!://).)*$|" + loc.protocol + "//" + loc.host);
        return url.match(reg);
      }

      // gather all resources
      var scripts = document.getElementsByTagName("script"),
          links = document.getElementsByTagName("link"),
          uris = [];

      // track local js urls
      for (var i = 0; i < scripts.length; i++) {
        var script = scripts[i], src = script.getAttribute("src");
        if (src && isLocal(src))
          uris.push(src);
        if (src && src.match(/\blive.js#/)) {
          for (var type in active)
            active[type] = src.match("[#,|]" + type) != null
          if (src.match("notify"))
            alert("Live.js is loaded.");
        }
      }
      if (!active.js) uris = [];
      if (active.html) uris.push(document.location.href);

      // track local css urls
      for (var i = 0; i < links.length && active.css; i++) {
        var link = links[i], rel = link.getAttribute("rel"), href = link.getAttribute("href", 2);
        if (href && rel && rel.match(new RegExp("stylesheet", "i")) && isLocal(href)) {
          uris.push(href);
          currentLinkElements[href] = link;
        }
      }

      // initialize the resources info
      for (var i = 0; i < uris.length; i++) {
        var url = uris[i];
        Live.getHead(url, function (url, info) {
          resources[url] = info;
        });
      }

      // add rule for morphing between old and new css files
      var head = document.getElementsByTagName("head")[0],
          style = document.createElement("style"),
          rule = "transition: all .3s ease-out;"
      css = [".livejs-loading * { ", rule, " -webkit-", rule, "-moz-", rule, "-o-", rule, "}"].join('');
      style.setAttribute("type", "text/css");
      head.appendChild(style);
      style.styleSheet ? style.styleSheet.cssText = css : style.appendChild(document.createTextNode(css));

      // yep
      loaded = true;
    },

    // check all tracking resources for changes
    checkForChanges: function () {
      for (var url in resources) {
        if (pendingRequests[url])
          continue;

        Live.getHead(url, function (url, newInfo) {
          var oldInfo = resources[url],
              hasChanged = false;
          resources[url] = newInfo;
          for (var header in oldInfo) {
            // do verification based on the header type
            var oldValue = oldInfo[header],
                newValue = newInfo[header],
                contentType = newInfo["Content-Type"];
            switch (header.toLowerCase()) {
              case "etag":
                if (!newValue) break;
                // fall through to default
              default:
                hasChanged = oldValue != newValue;
                break;
            }
            // if changed, act
            if (hasChanged) {
              Live.refreshResource(url, contentType);
              break;
            }
          }
        });
      }
    },

    // act upon a changed url of certain content type
    refreshResource: function (url, type) {
      switch (type.toLowerCase()) {
        // css files can be reloaded dynamically by replacing the link element
        case "text/css":
          var link = currentLinkElements[url],
              html = document.body.parentNode,
              head = link.parentNode,
              next = link.nextSibling,
              newLink = document.createElement("link");

          html.className = html.className.replace(/\s*livejs\-loading/gi, '') + ' livejs-loading';
          newLink.setAttribute("type", "text/css");
          newLink.setAttribute("rel", "stylesheet");
          newLink.setAttribute("href", url + "?now=" + new Date() * 1);
          next ? head.insertBefore(newLink, next) : head.appendChild(newLink);
          currentLinkElements[url] = newLink;
          oldLinkElements[url] = link;

          // schedule removal of the old link
          Live.removeoldLinkElements();
          break;

        // check if an html resource is our current url, then reload
        case "text/html":
          if (url != document.location.href)
            return;

          // local javascript changes cause a reload as well
        case "text/javascript":
        case "application/javascript":
        case "application/x-javascript":
          document.location.reload();
      }
    },

    // removes the old stylesheet rules only once the new one has finished loading
    removeoldLinkElements: function () {
      var pending = 0;
      for (var url in oldLinkElements) {
        // if this sheet has any cssRules, delete the old link
        try {
          var link = currentLinkElements[url],
              oldLink = oldLinkElements[url],
              html = document.body.parentNode,
              sheet = link.sheet || link.styleSheet,
              rules = sheet.rules || sheet.cssRules;
          if (rules.length >= 0) {
            oldLink.parentNode.removeChild(oldLink);
            delete oldLinkElements[url];
            setTimeout(function () {
              html.className = html.className.replace(/\s*livejs\-loading/gi, '');
            }, 100);
          }
        } catch (e) {
          pending++;
        }
        if (pending) setTimeout(Live.removeoldLinkElements, 50);
      }
    },

    // performs a HEAD request and passes the header info to the given callback
    getHead: function (url, callback) {
      pendingRequests[url] = true;
      var xhr = window.XMLHttpRequest ? new XMLHttpRequest() : new ActiveXObject("Microsoft.XmlHttp");
      xhr.open("HEAD", url, true);
      xhr.onreadystatechange = function () {
        delete pendingRequests[url];
        if (xhr.readyState == 4 && xhr.status != 304) {
          xhr.getAllResponseHeaders();
          var info = {};
          for (var h in headers) {
            var value = xhr.getResponseHeader(h);
            // adjust the simple Etag variant to match on its significant part
            if (h.toLowerCase() == "etag" && value) value = value.replace(/^W\//, '');
            if (h.toLowerCase() == "content-type" && value) value = value.replace(/^(.*?);.*?$/i, "$1");
            info[h] = value;
          }
          callback(url, info);
        }
      }
      xhr.send();
    }
  };

  // start listening
  if (document.location.protocol != "file:") {
    if (!window.liveJsLoaded)
      Live.heartbeat();

    window.liveJsLoaded = true;
  }
  else if (window.console)
    console.log("Live.js doesn't support the file protocol. It needs http.");
})();
'''


# -*- coding: utf8 -*-
"""
Icecake

This module provides a simple static site builder, similar to pelican or
octopress. It is intended to be small, light, and easy to modify. Out of the box
it sports the following features:

- Markdown formatting
- Pygments source code highlighting
- Jinja templates
- Automatically rebuilds when you edit things
"""
from __future__ import absolute_import, division, print_function, unicode_literals
import codecs
import logging
import os
import platform
import posixpath
from os.path import abspath, basename, dirname, exists, isdir, isfile, join, normpath, relpath, splitext
from multiprocessing import Process
import time
import shutil


import click
import jinja2
import jinja2.meta
import markdown
from dateutil.parser import parse as dateparse
from werkzeug.contrib.atom import AtomFeed
import watchdog.observers
import watchdog.events


from .templates import templates
from .livejs import livejs
if platform.python_version_tuple()[0] == '2':
    import ConfigParser as configparser
    import io
    from SimpleHTTPServer import SimpleHTTPRequestHandler
    from SocketServer import TCPServer
else:
    import configparser
    from http.server import SimpleHTTPRequestHandler
    from socketserver import TCPServer


__metaclass__ = type
logging.basicConfig(level=logging.ERROR)
click.disable_unicode_literals_warning = True
curdir = abspath(os.getcwd())


def ui(*args, **kwargs):
    pass


def ls_relative(list_path):
    """
    List files relative to the specified path
    """
    found = []
    # Guard against dir doesn't exist
    if not isdir(list_path):
        return found
    for path, dirs, files in os.walk(list_path):
        for file in files:
            filepath = join(path, file)
            found.append(relpath(filepath, list_path))
    found.sort()  # Make sure the sort order is deterministic
    return found


class ContentCache:
    def __init__(self, root):
        self.root = root
        self.files = {}
        self.pages = {}
        self.templates = {}
        self.rebuild_index = {}

    def peek(self, filename):
        """
        peek when you want to get fresh data from disk but NOT store it in the cache
        """
        file = join(self.root, filename)
        if isfile(file):
            content = codecs.open(file, encoding='utf-8').read()
            return content
        return None

    def read(self, filename):
        """
        read when you want to get fresh data from disk and store it in the cache
        """
        content = self.peek(filename)
        if content is not None:
            self.set(filename, content)
        return content

    def set(self, filename, content):
        if filename.startswith('content'):
            # Markdown files are not templates so let's skip those
            if splitext(filename)[1] != '.md':
                self.templates[relpath(filename, 'content')] = content
        if filename.startswith('layouts'):
            self.templates[relpath(filename, 'layouts')] = content
        self.files[filename] = content

    def get(self, filename):
        if filename in self.files:
            return self.files[filename]
        return None

    def delete(self, filename):
        del(self.files[filename])

    def move(self, old, new):
        if old not in self.files:
            return
        self.set(new, self.get(old))
        self.delete(old)

    def warm(self):
        for path in ['content', 'layouts']:
            for file in ls_relative(join(self.root, path)):
                self.read(join(path, file))


class Page:
    """
    A page is any discrete piece of content that will appear in your output
    folder. At minimum a page should have a body, title, and slug so it can be
    rendered. However, a page may have additional metadata like date or tags.
    """
    metadata = ["tags", "date", "title", "slug", "template"]
    required = ["date", "title"]
    metadelimiter = "++++"

    def __init__(self, filepath, site):
        # These are set when the page is initialized (step 1)
        self.parsed = False  # This is a special flag that helps us avoid bugs
        self.site = site
        self.abspath = abspath(filepath)
        # This is the path of the file relative to content
        self.filepath = relpath(filepath, join(site.root, 'content'))
        self.folder = self._get_folder()      # This is the leading part of the URL
        self.slug = self._get_default_slug()  # This is the final part of the URL
        self.ext = self._get_extension()      # This is the extension (markdown or html)

        # Note! We are going to evaluate this now to make sure it is always
        # evaluated. However, we will evaluate this again after the metadata is
        # parsed in case the user specifies a custom slug, which will change the
        # url. This way, though, we always have url defined even for pages
        # without metadata.
        self.url = self._get_url()           # This is the URL path

        # These maybe set when the page is parsed for metadata (step 2)
        self.date = None      # This is the date the page was published
        self.tags = None      # This is a list of tags, used to build links
        self.template = None  # This is the template we'll use to render the page
        self.title = None     # This is the title of the page

        # These are set when the page is rendered (step 3)
        self.body = None      # This is the raw body of the page
        self.content = None   # This is the content string for markdown pages
        self.rendered = None  # This is the HTML content of the page

    def _get_folder(self):
        return dirname(self.filepath)

    def _get_default_slug(self):
        """
        Get the slug for this page, which is used for the final part of the URL
        path. For a file like cakes/chocolate.html this will be "chocolate". If
        the slug is "index" we special case to "" so index.md and index.html can
        be used as you would expect without going into a folder called "index".
        If you want a folder called "index" you can specify the slug manually.
        """
        slug = splitext(basename(self.filepath))[0]
        if slug == "index":
            slug = ""
        return slug

    def _get_extension(self):
        """
        This is the file's extention, which is primarily used to identify
        markdown files.
        """
        return splitext(self.filepath)[1]

    def _get_url(self):
        """
        Get the url for this page, based on its filepath. This is path + slug.
        Something like cakes/chocolate.html will become /cakes/chocolate/
        """
        if self.ext in [".html", ".md", ".markdown"]:
            return '/%s/' % normpath(join(self.folder, self.slug))
        return '/%s' % join(self.folder, self.slug + self.ext)

    def get_target(self):
        """
        Get the target filename for this page. This is the containing folder +
        slug, which may have been customized.
        """
        # Note that we may have customized slug so we should only run this after
        # metadata has been parsed.
        if not self.parsed:
            raise RuntimeError("This should not be called before metadata is parsed")
        # If we find a .html or .md we will use .html instead, and will convert
        # the name to a folder containing index.html so we get a clean URL.
        if self.ext in [".html", ".md", ".markdown"]:
            return normpath(join(self.folder, self.slug, "index.html"))
        # We default to path/name.extension in case there is a file like CSS or
        # XML where the filename is important.
        return normpath(join(self.folder, self.slug + self.ext))

    def parse_metadata(self, text):
        """
        Parse a metadata string into object properties tags, date, title, etc.
        """
        logging.debug("Parsing metadata %s", text)
        text = "[Metadata]\n" + text
        parser = configparser.ConfigParser()
        if platform.python_version_tuple()[0] == '2':
            parser.readfp(io.StringIO(text))
        else:
            parser.read_string(text)

        # This is super ugly because of 2/3 compat. There's probably a cleaner
        # way to factor this code.
        values = {}
        for k, v in parser.items("Metadata"):
            values[k] = v
        for key in self.metadata:
            value = None
            if key in values:
                value = values[key]
            if key == "tags":
                if value is not None:
                    value = value.split(" ")
                else:
                    value = []
            # If we have a default metadata value and the user did not override
            # it, leave it alone. Currently this only applies to slug, which has
            # a default based on the filepath.
            if value is None and getattr(self, key, None) is not None:
                continue
            setattr(self, key, value)
            if value is None and key in self.required:
                logging.warning("Metadata '%s' not specified in %s", key,
                                self.filepath)
        self.url = self._get_url()
        self.parsed = True

    def render(self):
        """
        Render the page. All files will be rendered using Jinja. Markdown files
        with .md or .markdown extension will also use the markdown renderer. By
        default the base template is markdown.html for markdown and basic.html
        for everything else. Customize this via the "template" metadata field.
        """
        logging.debug("Rendering %s" % self.filepath)
        if self.ext in [".md", ".markdown"]:
            self.content = markdown.markdown(self.body,
                                             extensions=self.site.markdown_plugins,
                                             extension_configs=self.site.markdown_options)
            if self.template is not None:
                template = self.site.renderer.get_template(self.template)
            else:
                template = self.site.renderer.get_template("markdown.html")
        else:
            template = self.site.renderer.get_template(self.filepath)
        # Inject livejs code (optional)
        if self.site.preview_mode:
            livejs_code = "<script>"+livejs+"</script>"
        else:
            livejs_code = ""
        self.rendered = template.render(self.__dict__, site=self.site, livejs=livejs_code)
        return self.rendered

    def render_to_disk(self):
        output = self.render()
        target = join(self.site.root, 'output', self.get_target())
        logging.debug('Writing to %s' % target)
        ui('Generating %s' % target)
        target_dir = dirname(target)
        if not isdir(target_dir):
            os.makedirs(target_dir)
        file = codecs.open(target, encoding='utf-8', mode='w')
        file.write(output)
        file.close()

    @classmethod
    def parse_string(cls, filepath, site, text):
        """
        Parse a raw string and separate the front matter so we can turn it into
        a page object with metadata and body.
        """
        page = cls(filepath, site)
        parts = text.split(cls.metadelimiter, 1)

        if len(parts) == 2:
            page.parse_metadata(parts[0].strip())
            page.body = parts[1].strip()
        else:
            page.body = parts[0].strip()
            if page.ext in ['.md', '.markdown']:
                logging.warning("No metadata detected; expected %s separator %s",
                                cls.metadelimiter, page.filepath)
            # Say we already parsed the metadata
            page.parsed = True
        return page

    @classmethod
    def parse_file(cls, filepath, site):
        """
        Read a file and return the Page created by passing it into parse_string
        """
        content = codecs.open(filepath, encoding='utf-8').read()
        page = cls.parse_string(filepath, site, content)
        return page


class Site:
    """
    A site represents a collection of source files arranged under the following
    project structure:

    ├── content
    ├── layouts
    ├── static
    └── output

    Content represent pages on your site. Each page will be built into a
    corresponding file in your output directory. Layout files are used for
    shared templates but do not have a corresponding page in output. Files under
    static are copied directly, and is a good place to put images, css, and
    javascript.

    Methods on this class deal with configuration, discovering content and
    building your site.
    """

    def __init__(self, root, preview_mode=False):
        """
        Keyword Arguments:
        root -- The path to the static site folder which includes the pages,
                layouts, and static folders.
        """
        self.preview_mode = preview_mode
        self.root = abspath(root)
        self.cache = ContentCache(root)
        self.cache.warm()
        self.markdown_plugins = ["markdown.extensions.fenced_code", "markdown.extensions.codehilite"]
        self.markdown_options = {
            "codehilite": {
                "linenums": False,
                "guess_lang": False,
            }
        }
        self.renderer = jinja2.Environment(loader=jinja2.DictLoader(self.cache.templates))
        self.pagedata = {}
        self.get_pages()

    def get_target(self, path):
        """Convert a path from static to output"""
        path = self.relpath(path)
        if path.startswith('static'):
            return join(self.root, 'output', relpath(path, 'static'))
        raise ValueError('Invalid path %s; expected a path under static')

    def relpath(self, path):
        """Get path relative to the site root"""
        return relpath(join(self.root, getattr(path, 'src_path', path)), self.root)

    def is_content(self, path):
        return self.relpath(path).startswith('content')

    def is_layout(self, path):
        return self.relpath(path).startswith('layouts')

    def is_static(self, path):
        return self.relpath(path).startswith('static')

    def copy_static(self, path):
        source = join(self.root, 'static', path)
        target = self.get_target(source)
        target_dir = dirname(target)
        if not isdir(target_dir):
            logging.debug('Creating directory %s' % target_dir)
            os.makedirs(target_dir, mode=0o755)
        logging.debug('Copying static file to %s' % target)
        shutil.copy(source, target)

    def copy_all_static(self):
        logging.debug('Copying static files')
        static_dir = join(self.root, 'static')
        sources = ls_relative(static_dir)
        for source in sources:
            if isfile(join(self.root, 'static', source)):
                self.copy_static(source)

    def get_pages(self):
        """
        Enumerate and parse all the page files in the static site.
        """
        logging.debug("Getting pages")
        pages = {}
        for file in self.cache.files:
            if not file.startswith('content'):
                continue
            source_file = join(self.root, file)
            if isfile(source_file):
                logging.debug("Parsing %s", source_file)
                page = Page.parse_file(source_file, self)
                page.render()
                pages[page.filepath] = page
        self.pagedata = pages
        return self.pagedata

    def list_dependents(self, filepath):
        depset = set()
        # If the page is markdown.html then we need to add all of the markdown
        # pages.
        # TODO add support for markdown pages that specify a custom template
        if filepath == 'markdown.html':
            for _, page in self.pagedata.items():
                if page.ext in ['.md', '.markdown']:
                    depset.add(page.filepath)
        else:
            for path, body in self.cache.templates.items():
                ast = self.renderer.parse(body)
                pagedeps = jinja2.meta.find_referenced_templates(ast)
                if filepath in pagedeps:
                    for item in self.list_dependents(path):
                        depset.add(item)
            for _, page in self.pagedata.items():
                ast = self.renderer.parse(page.body)
                pagedeps = jinja2.meta.find_referenced_templates(ast)
                if filepath in list(pagedeps):
                    depset.add(page.filepath)
                    for item in self.list_dependents(page.filepath):
                        depset.add(item)
        deplist = list(depset)
        deplist.sort()
        return deplist

    def render_dependents(self, filepath):
        for item in self.list_dependents(filepath):
            self.pagedata[item].render_to_disk()

    def build(self):
        """
        Build the site. This method originates all of the calls to discover,
        render, and place pages in the output directory. If you want to
        customize how your site is built, this is a good place to start.
        """
        self.clean_output()
        self.pagedata = self.get_pages()
        for _, page in self.pagedata.items():
            page.render_to_disk()
        self.copy_all_static()

    def tags(self):
        tagnames = set()
        for path, page in self.pagedata.items():
            if page.tags:
                tagnames = tagnames.union(set(page.tags))
        taglist = list(tagnames)
        taglist.sort()
        return taglist

    def pages(self, path=None, tag=None, limit=None, order=None):
        """
        Filter the pages on your site by path or tag, and (optionally) sort or
        limit the number of results. Path pages uses startswith. Tag pages
        uses exact match. Use -ORDER for reverse sort. Examples:

        finder.pages(path="articles", limit=5, order="-date")
        finder.pages(tag="family", order="title")
        """
        items = self.pagedata.values()
        for page in items:
            logging.debug(page.filepath)
        if path is not None:
            items = [page for page in items if page.filepath.startswith(path)]
        if tag is not None:
            items = [page for page in items if tag in page.tags]
        if order is not None:
            rev = False
            if order[0] == "-":
                rev = True
                order = order[1:]
            items.sort(key=lambda x: getattr(x, order), reverse=rev)
        if limit is not None and limit > 0:
            items = items[:limit]
        return items

    def atom(self, feed_title, feed_url, feed_subtitle, site_url, author, *args, **kwargs):
        items = self.pages(*args, **kwargs)

        atom = AtomFeed(title=feed_title,
                        subtitle=feed_subtitle,
                        feed_url=feed_url,
                        url=site_url)
        for item in items:
            item.render()
            atom.add(title=item.title,
                     content=item.content,
                     content_type='html',
                     author=author,
                     url=site_url+item.url,
                     published=dateparse(item.date),
                     updated=dateparse(item.date),
                     xml_base=None)
        return atom.to_string()

    def clean_output(self):
        """
        Delete everything in the output folder so we can perform a clean build
        """
        output_dir = join(self.root, 'output')
        if isdir(output_dir):
            shutil.rmtree(output_dir)

    @classmethod
    def initialize(cls, root):
        if not isdir(root):
            os.makedirs(root)
        for path, contents in templates.items():
            target = join(root, path)
            target_dir = dirname(target)
            if not isdir(target_dir):
                os.makedirs(target_dir)

            with codecs.open(target, encoding='utf-8', mode="w") as f:
                logging.debug("Writing %s" % target)
                f.write(contents)
                f.close()
        return Site(root)


class Handler(watchdog.events.FileSystemEventHandler):
    site = None

    def is_watched(self, event):
        """
        Whether we are watching this path at all. This guards against
        triggering logic on the output folder or other folders the user may
        have created here.
        """
        return self.site.is_content(event) or self.site.is_layout(event) or self.site.is_static(event)

    def on_created(self, event):
        if isfile(event.src_path):
            if self.site.is_content(event):
                data = self.site.cache.read(event.src_path)
                page = Page.parse_string(event.src_path, self.site, data)
                page.render_to_disk()
            elif self.site.is_static(event):
                self.site.copy_static(event.src_path)

    def on_deleted(self, event):
        if self.is_watched(event.src_path):
            logging.debug('Deletion detected for %s', event.src_path)
            shutil.rmtree(event.src_path)

    def on_modified(self, event):
        if isfile(event.src_path) and self.is_watched(event):
            path = self.site.relpath(event.src_path)
            logging.debug('Change detected for %s', event.src_path)
            if self.site.is_content(event):
                if self.site.cache.get(path) != self.site.cache.read(path):
                    data = self.site.cache.get(path)
                    page = Page.parse_string(path, self.site, data)
                    page.render_to_disk()
                    self.site.render_dependents(relpath(self.site.relpath(event.src_path), 'content'))
            elif self.site.is_static(event):
                self.site.copy_static(event.src_path)
            elif self.site.is_layout(event):
                if self.site.cache.get(path) != self.site.cache.read(path):
                    self.site.render_dependents(relpath(self.site.relpath(event.src_path), 'layouts'))

    def on_moved(self, event):
        if isfile(event.dest_path) and self.is_watched(event):
            self.on_deleted(event)
            # This is a hack and probably has some bad consequence I'm not
            # thinking about right now.
            event.src_path = event.dest_path
            self.on_created(event)


class Watcher:
    def __init__(self, site):
        self.site = site
        Handler.site = site

    def watch(self):
        obs = watchdog.observers.Observer()
        obs.schedule(Handler(), join(self.site.root), recursive=True)
        logging.debug('Watching for changes in %s' % self.site.root)
        ui('Watching for changes in %s' % self.site.root)
        obs.start()
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            obs.stop()
        obs.join()


class HTTPHandler(SimpleHTTPRequestHandler):
    site = None

    def guess_type(self, path):
        base, ext = posixpath.splitext(path)
        if ext == '.html':
            return 'text/html; charset=utf-8'
        if ext == '.css':
            return 'text/css; charset=utf-8'
        if ext == '.js':
            return 'text/javascript; charset=utf-8'
        if platform.python_version_tuple()[0] == '2':
            return SimpleHTTPRequestHandler.guess_type(self, path)
        else:
            return super().guess_type(path)


    def translate_path(self, path):
        if platform.python_version_tuple()[0] == '2':
            path = SimpleHTTPRequestHandler.translate_path(self, path)
        else:
            path = super().translate_path(path)
        # Since we are serving from the project root instead of content root
        # we want to insert 'content' into the filepath.
        path = abspath(join('output', relpath(path)))
        return path

    def log_request(self, code='-', size='-'):
        # Don't log HEAD requests because these are very spammy with livejs turned on
        if self.command == 'HEAD':
            return
        if platform.python_version_tuple()[0] == '2':
            SimpleHTTPRequestHandler.log_request(self, code, size)
        else:
            super().log_request(code, size)


class HTTPServer(TCPServer):
    def server_activate(self):
        ui('Server started successfully')
        logging.debug('Listening on http://%s:%s/' % self.server_address)
        if platform.python_version_tuple()[0] == '2':
            TCPServer.server_activate(self)
        else:
            super().server_activate()


class Server:
    def __init__(self, site):
        self.site = site

    def serve(self, address, port):
        HTTPHandler.site = self.site
        ui('Starting server on http://%s:%s/' % (address, port))
        ui('HEAD requests are omitted from the logs')
        httpd = None
        while True:
            try:
                httpd = HTTPServer((address, port), HTTPHandler)
                httpd.serve_forever()
            except OSError:
                ui('ERROR: Listen socket is busy; will retry in 5 seconds')
                time.sleep(5)
            except KeyboardInterrupt:
                httpd.shutdown()
                break


@click.group()
def cli():
    global ui
    ui = click.echo


@cli.command(help="""
    Initialize a project in the specified directory. The path will be created if
    it does not exist.
    """)
@click.option("--debug/--no-debug", default=False)
@click.option("-f/--force", help="Initialize even if the directory is not empty", default=False)
@click.argument("path", type=click.Path())
def init(debug, f, path):
    if debug:
        logging.getLogger().setLevel(logging.DEBUG)
    if len(ls_relative(path)) > 0 and not f:
        click.echo("Path \"%s\" already contains files; use -f to force initialization" % path)
        exit(1)
    Site.initialize(path)


@cli.command()
@click.option("--debug/--no-debug", default=False)
def build(debug):
    if debug:
        logging.getLogger().setLevel(logging.DEBUG)
    Site(curdir).build()


@cli.command()
@click.option("--debug/--no-debug", default=False)
@click.option("--address", '-a', default="127.0.0.1", type=str)
@click.option("--port", '-p', default=8000, type=int)
def preview(debug, address, port):
    if debug:
        logging.getLogger().setLevel(logging.DEBUG)

    site = Site(curdir, preview_mode=True)
    site.build()

    watcher = Watcher(site)
    watcher_pid = Process(target=watcher.watch)
    watcher_pid.daemon = False
    watcher_pid.start()

    server = Server(site)
    server_pid = Process(target=server.serve, args=(address, port))
    server_pid.daemon = False
    server_pid.start()

    click.echo('Use Ctrl-C to quit')

    watcher_pid.join()
    server_pid.join()


@cli.command()
@click.option("--debug/--no-debug", default=False)
def watch(debug):
    if debug:
        logging.getLogger().setLevel(logging.DEBUG)
    Watcher(Site(curdir, preview_mode=True)).watch()


@cli.command()
@click.option("--debug/--no-debug", default=False)
def serve(debug):
    if debug:
        logging.getLogger().setLevel(logging.DEBUG)
    Server(Site(curdir, preview_mode=True)).serve("127.0.0.1", 8000)


if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-

templates = {
"content/articles.html":
u"""{% extends "basic.html" %}

{% block title %}Articles :: My Site.com{% endblock %}

{% block content %}

<div class="article">

<h1>Articles</h1>

<ul class="articles">
{% for page in site.pages(path="articles/", order="-date") %}
	<li><a href="{{ page.url }}">{{ page.title }}</a></li>
{% endfor %}
</ul>

</div>

{% endblock %}
""",
"content/articles/hello-world.md":
u"""title = Hello world!
date = 2016-04-02
tags = hello
++++

I'm building a website! When I do something cool that I'd like to share with you, I'm going to write about it here.

I'll write with [Markdown](https://daringfireball.net/projects/markdown/) and maybe even write some code:

```
# This is a title

And this is a list:

- cake
- chocolate
- candy
```

Here are some unicode characters: 😀 Ω 屋. That's all for now!
""",
"content/atom.xml":
u"""{{
  site.atom(
    path="articles/",
    order="-date",
    site_url="https://example.com",
    feed_url="https://example.com"+url,
    feed_title="My Site",
    feed_subtitle=None,
    author="a smart person"
  )
}}""",
"content/index.html":
u"""{% extends "basic.html" %}

{% block title %}My Site.com{% endblock %}

{% block content %}

<div class="article">

<p>Hi, this is my site.</p>

<p>Here's something interesting about me that you might not know.</p>

<p>Follow me on <a href="https://twitter.com/">Twitter</a></p>

<h2>Recent Articles</h2>
<ul class="articles">
{% for page in site.pages(path="articles/", limit=5, order="-date") %}
	<li><a href="{{ page.url }}">{{ page.title }}</a></li>
{% endfor %}
</ul>

<p><a href="/articles/">See all of them</a></p>

</div>

{% endblock %}
""",
"content/tags.html":
u"""{% extends "basic.html" %}

{% block title %}My Site.com{% endblock %}

{% block content %}

<div class="article">

{% for tag in site.tags() %}
	<div id="tag-{{tag}}" class="tag-block">
	<h1>{{tag}}</h1>
	<ul>
	{% for page in site.pages(path="articles/", order="title", tag=tag) %}
		<li><a href="{{ page.url }}">{{ page.title }}</a></li>
	{% endfor %}
	</ul>
	</div>
{% endfor %}

</div>

{% endblock %}
""",
"layouts/basic.html":
u"""<!DOCTYPE html>
<head>
  <title>{% block title %}{{ title }} :: My Site.com{% endblock %}</title>
  <link href="/css/main.css" rel="stylesheet" />
  <link href="/css/syntax.css" rel="stylesheet" />
  <link href="/atom.xml" rel="alternate" type="application/atom+xml" title="My Site" />
  <link href="//fonts.googleapis.com/css?family=Domine:400,700" rel="stylesheet" type="text/css" />
</head>
<body>

<nav id="main-nav">
  <a href="/">My Site</a>
  <a href="/articles/">Articles</a>
</nav>

<section id="main-content">
{% block content %}{% endblock %}
</section>

<footer id="main-footer">
  <p>Copyright &copy; 2016 A Smart Person. All rights reserved. | Proudly built with <a href="https://github.com/cbednarski/icecake">Icecake</a></p>
</footer>
{{ livejs }}
</body>
</html>
""",
"layouts/markdown.html":
u"""{% extends "basic.html" %}

{% block title %}{{ title }} :: My Site.com{% endblock %}

{% block content %}

<div class="article">
  <h1 class="title">{{ title }}</h1>
  <p class="date">Published {{ date }}</p>

{{ content }}

<hr>

<h4>Related</h4>

<p>
{% for tag in tags %}
<a class="tag" href="/tags/#tag-{{ tag }}">{{ tag }}</a>
{% endfor %}
</p>

</div>

{% endblock %}
""",
"static/css/main.css":
u"""body {
    background-color: #05b3e3;
    color: #000;
    font-family: Domine, "PT Serif", Georgia, "Times New Roman", serif;
    font-size: 18px;
    margin: 0;
}

h1, h2, h3, h4, h5 ,h6 {
    color: #444;
}

hr {
    border: 1px solid #777;
    margin:1.5em 0;
}

code,pre {
    font-family: Menlo, Monaco, "Source Sans Pro", "Ubuntu Mono", Consolas, "Courier New", monospace;
    background-color: #F5F5F5;
}

pre {
    font-size: .8em;
    line-height: 1.4em;
    padding: .5em;
    border-radius: 5px;
    overflow-wrap: break-word;
}

code {
    font-size: .8em;
    padding: 1px 4px;
    border-radius: 5px;
    overflow-wrap: break-word;
}

pre>code{
    padding: 0;
    background: transparent;
}

a {
    color: #37AFFF;
    text-decoration: none;
}

blockquote {
    font-style: italic;
    border-left: 4px solid #777;
    color: #555;
    margin: 0;
    padding: .01em 2em;
    background-color: #eee;
}

a:hover, #menu a:hover {
    border-bottom: 1px solid;
}

.box {
    background-color: #fff;
    color: #555;
}

.clear {
    clear: both;
}

.article {
    background: #fff;
    color: #000;
    margin: auto;
    line-height: 2em;
    max-width: 40em;
    padding: 100px;
    margin-top: 50px;
}

.article img {
    max-width: 100%;
    max-height: 100%;
}

.article h1.title {
    margin-top: 0;
    font-size: 2em;
}

.article h1.comments {
    margin-top: 3em;
}

.article-heading h2 a{
    display: block;
    padding: 20px 30px;
    margin-bottom: 20px;
    background-color: #fff;
    transition: background .3s;
    border: none;
}

.article-heading h2 a:hover {
    background-color: #ddd;
    text-decoration: none;
}

.article .date {
    color: #aaa;
    font-style: italic;
}

.chaser h3 {
    margin-top: .7em;
    margin-bottom: .5em;
}

.chaser a {
    margin-right: .2em;
}

.datetime {
    font-style: italic;
    color: #bbb;
    font-size: .8em;
}

.prevnext {
    width: 100%;
}

.prevnext>div {
    width: 48%;
    background-color: #333;
    color: #fff;
}

.prevnext>div a {
    display: block;
    padding: 10px;
    border-bottom: 0;
}

.prevnext>div a:hover {
    color: #80CC15;
}

.prevnext .prev {
    float: left;
}

.prevnext .next {
    float: right;
}

.tag {
    padding: 0 .2em;
}

.tag-block {
    padding: 5px;
    border-radius: 5px;
    margin-bottom: 10px;
    border: 1px solid #fff;
}

.tag-block h2 {
    margin-top: 0;
}

ul.articles, .tag-block ul {
    margin: 0;
    padding: 0;
}

ul.articles li, .tag-block li {
    list-style: none;
}

.tag-block:target {
    border: 1px dashed #37AFFF;
}

#main-nav {
    height: 3em;
    font-size: 1.4em;
    line-height: 3em;
    color: #000;
    padding-left: 2.5em;
}

#main-nav a {
    border: 0;
    color: #000;
    display: inline-block;
    padding: 6px 10px;
}

#main-nav a:hover {
    color: #fff;
    text-decoration: none;
}

#main-footer {
    background-color: transparent;
    clear: both;
    color: #555;
    display: block;
    height: inherit;
    margin: 60px 0 30px;
    text-align: middle;
    vertical-align: bottom;
    width: 100%;
    z-index: 1;
    font-size: .8em;
}

#main-footer p {
    text-align: center;
}

#main-footer a {
    color: #000;
    border-bottom: 1px solid #000;
}
""",
"static/css/syntax.css":
u""".codehilite .hll { background-color: #ffffcc }
.codehilite .c { color: #0099FF; font-style: italic } /* Comment */
.codehilite .err { color: #AA0000; background-color: #FFAAAA } /* Error */
.codehilite .k { color: #006699; font-weight: bold } /* Keyword */
.codehilite .o { color: #555555 } /* Operator */
.codehilite .cm { color: #0099FF; font-style: italic } /* Comment.Multiline */
.codehilite .cp { color: #009999 } /* Comment.Preproc */
.codehilite .c1 { color: #0099FF; font-style: italic } /* Comment.Single */
.codehilite .cs { color: #0099FF; font-weight: bold; font-style: italic } /* Comment.Special */
.codehilite .gd { background-color: #FFCCCC; border: 1px solid #CC0000 } /* Generic.Deleted */
.codehilite .ge { font-style: italic } /* Generic.Emph */
.codehilite .gr { color: #FF0000 } /* Generic.Error */
.codehilite .gh { color: #003300; font-weight: bold } /* Generic.Heading */
.codehilite .gi { background-color: #CCFFCC; border: 1px solid #00CC00 } /* Generic.Inserted */
.codehilite .go { color: #AAAAAA } /* Generic.Output */
.codehilite .gp { color: #000099; font-weight: bold } /* Generic.Prompt */
.codehilite .gs { font-weight: bold } /* Generic.Strong */
.codehilite .gu { color: #003300; font-weight: bold } /* Generic.Subheading */
.codehilite .gt { color: #99CC66 } /* Generic.Traceback */
.codehilite .kc { color: #006699; font-weight: bold } /* Keyword.Constant */
.codehilite .kd { color: #006699; font-weight: bold } /* Keyword.Declaration */
.codehilite .kn { color: #006699; font-weight: bold } /* Keyword.Namespace */
.codehilite .kp { color: #006699 } /* Keyword.Pseudo */
.codehilite .kr { color: #006699; font-weight: bold } /* Keyword.Reserved */
.codehilite .kt { color: #007788; font-weight: bold } /* Keyword.Type */
.codehilite .m { color: #FF6600 } /* Literal.Number */
.codehilite .s { color: #CC3300 } /* Literal.String */
.codehilite .na { color: #330099 } /* Name.Attribute */
.codehilite .nb { color: #336666 } /* Name.Builtin */
.codehilite .nc { color: #00AA88; font-weight: bold } /* Name.Class */
.codehilite .no { color: #336600 } /* Name.Constant */
.codehilite .nd { color: #9999FF } /* Name.Decorator */
.codehilite .ni { color: #999999; font-weight: bold } /* Name.Entity */
.codehilite .ne { color: #CC0000; font-weight: bold } /* Name.Exception */
.codehilite .nf { color: #CC00FF } /* Name.Function */
.codehilite .nl { color: #9999FF } /* Name.Label */
.codehilite .nn { color: #00CCFF; font-weight: bold } /* Name.Namespace */
.codehilite .nt { color: #330099; font-weight: bold } /* Name.Tag */
.codehilite .nv { color: #003333 } /* Name.Variable */
.codehilite .ow { color: #000000; font-weight: bold } /* Operator.Word */
.codehilite .w { color: #bbbbbb } /* Text.Whitespace */
.codehilite .mf { color: #FF6600 } /* Literal.Number.Float */
.codehilite .mh { color: #FF6600 } /* Literal.Number.Hex */
.codehilite .mi { color: #FF6600 } /* Literal.Number.Integer */
.codehilite .mo { color: #FF6600 } /* Literal.Number.Oct */
.codehilite .sb { color: #CC3300 } /* Literal.String.Backtick */
.codehilite .sc { color: #CC3300 } /* Literal.String.Char */
.codehilite .sd { color: #CC3300; font-style: italic } /* Literal.String.Doc */
.codehilite .s2 { color: #CC3300 } /* Literal.String.Double */
.codehilite .se { color: #CC3300; font-weight: bold } /* Literal.String.Escape */
.codehilite .sh { color: #CC3300 } /* Literal.String.Heredoc */
.codehilite .si { color: #AA0000 } /* Literal.String.Interpol */
.codehilite .sx { color: #CC3300 } /* Literal.String.Other */
.codehilite .sr { color: #33AAAA } /* Literal.String.Regex */
.codehilite .s1 { color: #CC3300 } /* Literal.String.Single */
.codehilite .ss { color: #FFCC33 } /* Literal.String.Symbol */
.codehilite .bp { color: #336666 } /* Name.Builtin.Pseudo */
.codehilite .vc { color: #003333 } /* Name.Variable.Class */
.codehilite .vg { color: #003333 } /* Name.Variable.Global */
.codehilite .vi { color: #003333 } /* Name.Variable.Instance */
.codehilite .il { color: #FF6600 } /* Literal.Number.Integer.Long */""",
}


import codecs
import pytest
from icecake import cli
from icecake.templates import templates
import jinja2
from os.path import abspath, dirname, isdir, isfile, join


module_root = dirname(dirname(abspath(__file__)))
test_root = dirname(abspath(__file__))


class TestFunctions:
    def test_ls_relative(self):
        items = cli.ls_relative(join(test_root, 'fixtures', 'ls'))
        assert items == [
            'a.txt',
            'b.md',
            'c.html',
            'd/e.css'
        ]


class TestContentCache:
    def test_read(self):
        cache = cli.ContentCache(join(module_root, 'templates'))

        # Read from content
        helloworld = codecs.open(join(module_root, 'templates', 'content', 'articles', 'hello-world.md'), encoding='utf-8').read()
        assert cache.read('content/articles/hello-world.md') == helloworld
        assert cache.get('content/articles/hello-world.md') == helloworld
        cache.set('content/articles/hello-world.md', 'b')
        assert cache.get('content/articles/hello-world.md') == 'b'

        # Read from layouts
        basic = open(join(module_root, 'templates', 'layouts', 'basic.html')).read()
        assert cache.read('layouts/basic.html') == basic

        # Read missing file
        assert cache.read('layouts/nope.html') is None

    def test_missing(self):
        cache = cli.ContentCache(join(module_root, 'templates'))
        assert cache.get('nope') is None

    def test_delete(self):
        cache = cli.ContentCache(join(module_root, 'templates'))
        cache.set('content/pie.md', 'delicious')
        assert cache.get('content/pie.md') == 'delicious'
        cache.delete('content/pie.md')
        assert cache.get('content/pie.md') is None

    def test_move(self):
        cache = cli.ContentCache(join(module_root, 'templates'))
        cache.set('content/pie.md', 'delicious')
        cache.move('content/pie.md', 'content/cake.md')
        assert cache.get('content/pie.md') is None
        assert cache.get('content/cake.md') == 'delicious'

        # Doesn't exist; no-op
        cache.move('nope', 'yep')
        assert cache.get('yep') is None

    def test_warm(self):
        cache = cli.ContentCache(join(module_root, 'templates'))
        cache.warm()
        basic = open(join(module_root, 'templates', 'layouts', 'basic.html')).read()
        assert cache.get('layouts/basic.html') == basic


class TestPage:
    def test_init(self):
        site = cli.Site('.')
        page = cli.Page("./content/this/file/does/not/exist.html", site)
        assert page.slug == "exist"
        assert page.folder == "this/file/does/not"
        assert page.ext == ".html"
        assert page.url == "/this/file/does/not/exist/"

    def test_parse_checker(self):
        site = cli.Site('.')
        with pytest.raises(RuntimeError):
            page = cli.Page("./content/this/file/does/not/exist.html", site)
            page.get_target()

    def test_parse_metadata(self):
        meta = """
title = Some title
date = 2013-01-02
tags = pie cake chocolate
slug = some-title
template = myfile.html
        """

        site = cli.Site('.')
        page = cli.Page("./content/this/file/does/not/exist.html", site)
        page.parse_metadata(meta)

        assert page.title == "Some title"
        assert page.date == "2013-01-02"
        assert page.tags == ['pie', 'cake', 'chocolate']
        assert page.slug == "some-title"
        assert page.template == "myfile.html"
        assert page.url == "/this/file/does/not/some-title/"

    def test_get_target(self):
        # Test basic case
        site = cli.Site('.')
        page = cli.Page("./content/this/file/does/not/exist.html", site)
        page.parse_metadata("")
        assert page.get_target() == "this/file/does/not/exist/index.html"
        assert page.url == "/this/file/does/not/exist/"

        # Test not html / markdown case
        page = cli.Page("./content/this/file/does/not/exist.css", site)
        page.parse_metadata("")
        assert page.get_target() == "this/file/does/not/exist.css"
        assert page.url == "/this/file/does/not/exist.css"

        # Test index case
        page = cli.Page("./content/this/file/does/not/index.md", site)
        page.parse_metadata("")
        assert page.get_target() == "this/file/does/not/index.html"
        assert page.url == "/this/file/does/not/"

        # Test customized slug case
        meta = """
slug = some-title
        """
        page = cli.Page("./content/this/file/does/not/exist.md", site)
        page.parse_metadata(meta)
        assert page._get_folder() == "this/file/does/not"
        assert page.get_target() == "this/file/does/not/some-title/index.html"
        assert page.url == "/this/file/does/not/some-title/"

    def test_parse_string(self):
        raw = """
title = My Title
date = 2013-01-02
tags = pie
++++

This is the beginning of my page!
        """

        site = cli.Site('.')
        page = cli.Page.parse_string("this/file/does/not/exist.md", site, raw)
        assert page.body == "This is the beginning of my page!"

    def test_render(self):
        site = cli.Site(".")
        site.renderer = jinja2.Environment(loader=jinja2.DictLoader({
            "markdown.html": "MARKDOWN{{title}}{{content}}",
            "basic.html": "HTML{{content}}",
            "just.html": "Waka"
        }))

        page = cli.Page("filename.md", site)
        page.parse_metadata("title=My Title")
        page.body = "#This is awesome!"
        assert page.render() == "MARKDOWNMy Title<h1>This is awesome!</h1>"

        page.template = "basic.html"
        assert page.render() == "HTML<h1>This is awesome!</h1>"

        # This case is a bit odd. We don't actually use the page content we
        # parsed, we just let Jinja find the template on disk and render it
        # as-is. This is inconsistent from the markdown case, but works. There
        # may be some opportunity to clean up the API here and remove body or
        # content since only one is actually used.
        page = cli.Page("content/just.html", site)
        assert page.render() == "Waka"


class TestSite:
    def test_initialize(self, tmpdir):
        site = cli.Site.initialize(tmpdir.strpath)
        files = cli.ls_relative(site.root)
        # Verify all the things we expected are present
        for name in templates.keys():
            assert name in files
        # Verify we didn't find any extra stuff
        for file in files:
            assert templates[file] is not None

    def test_get_target(self):
        site = cli.Site(dirname(__file__))
        assert site.get_target('static/pie.css') == join(dirname(__file__), 'output/pie.css')

    def test_is_content(self):
        site = cli.Site(dirname(__file__))
        assert site.is_content('content/blah.html')
        assert not site.is_content('output/blah.html')

    def test_is_layout(self):
        site = cli.Site(dirname(__file__))
        assert site.is_layout('layouts/blah.html')
        assert not site.is_layout('output/blah.html')

    def test_is_static(self):
        site = cli.Site(dirname(__file__))
        assert site.is_static('static/blah.html')
        assert not site.is_static('output/blah.html')

    def test_copy_static(self, tmpdir):
        site = cli.Site.initialize(tmpdir.strpath)
        site.copy_static('css/main.css')
        target = join(site.root, 'output', 'css', 'main.css')
        assert isdir(join(site.root, 'output'))
        assert isdir(join(site.root, 'output', 'css'))
        assert isfile(target)

    def test_render(self, tmpdir):
        site = cli.Site.initialize(tmpdir.strpath)
        site.get_pages()
        page = site.pages(path='articles/hello-world.md')[0]
        output = page.render()
        # The source file has a trailing slash for git / unix reasons but this is dropped in the rendered version
        output += "\n"
        assert output == codecs.open(join(test_root, 'fixtures', 'render', 'hello-world.md.html'), encoding='utf-8').read()

    def test_build(self, tmpdir):
        site = cli.Site.initialize(tmpdir.strpath)
        assert isfile(join(site.root, 'content', 'articles.html'))

        site.build()
        output_files = cli.ls_relative(join(site.root, 'output'))

        # Make sure all the files are there
        assert output_files == [
            'articles/hello-world/index.html',
            'articles/index.html',
            'atom.xml',
            'css/main.css',
            'css/syntax.css',
            'index.html',
            'tags/index.html'
        ]

    def test_clean_output(self, tmpdir):
        site = cli.Site.initialize(tmpdir.strpath)
        files = cli.ls_relative(site.root)
        assert 'output/index.html' not in files

        site.build()
        files = cli.ls_relative(site.root)
        assert 'output/index.html' in files

        site.clean_output()
        files = cli.ls_relative(site.root)
        assert 'output/index.html' not in files


class TestUpdates:
    def test_list_dependents(self, tmpdir):
        site = cli.Site.initialize(tmpdir.strpath)
        items = site.list_dependents('markdown.html')
        assert items[0] == 'articles/hello-world.md'

        items = site.list_dependents('basic.html')
        items.sort()
        # We expect to find all of the content pages that need to be
        # re-rendered. We DO NOT expect to find markdown.html because
        # we are not going to render that page (it's a template) but
        # we DO expect to find articles/hello-world.md because markdown
        # is affected.
        assert items == [
            'articles.html',
            'articles/hello-world.md',
            'index.html',
            'tags.html',
        ]

    def test_render_dependents(self, tmpdir):
        site = cli.Site.initialize(tmpdir.strpath)
        site.render_dependents('markdown.html')
        files = cli.ls_relative(site.root)
        assert 'output/articles/hello-world/index.html' in files


class TestCLI:
    # TODO add tests for the CLI
    pass


class TestTemplates:
    """
    Verify that the templates in templates.py match the ones on disk
    """
    def test_templates(self):
        for f in cli.ls_relative(join(module_root, 'templates')):
            contents = codecs.open(join(module_root, 'templates', f), encoding='utf-8').read()
            assert templates[f] == contents



from setuptools import setup, find_packages
from codecs import open

setup(
    # packaging information that is likely to be updated between versions
    name='icecake',
    version='0.6.1',
    packages=['icecake'],
    py_modules=['cli', 'templates', 'livejs'],
    entry_points='''
        [console_scripts]
        icecake=icecake.cli:cli
    ''',
    install_requires=[
        'Click',
        'Jinja2',
        'Markdown',
        'Pygments',
        'python-dateutil',
        'watchdog',
        'Werkzeug',
    ],

    # pypy stuff that is not likely to change between versions
    url="https://github.com/cbednarski/icecake",
    author="Chris Bednarski",
    author_email="chris@cbednarski.com",
    description="An easy and cool static site generator",
    license="MIT",
    long_description=open('README.rst', encoding="utf-8").read(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
    ],
    keywords="static site generator builder icecake"
)
